
// JSonSerial2: C++ Object Serialization in JSON
// See: https://www.telecom-paris.fr/~elc/software/
// This file should be included *after* jsonserial.hpp when serializing std::vector
//
// Copyright 2020 Eric Lecolinet (eric.lecolinet@gmail.com)
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef jsonserial_vector_hpp
#define jsonserial_vector_hpp
#include <vector>
namespace jsonserial {

template <class T, class Alloc>
struct is_std_vector<std::vector<T,Alloc>> : std::true_type {};

template <class Alloc>
struct is_std_vector<std::vector<bool,Alloc>> : std::true_type {};

template<class T>
struct JSonArrayImpl<T, typename std::enable_if<is_std_vector<T>::value>::type>
: public JSonArray {
  struct Ptr {size_t index; JObjectPtr* jsp;};

  JSonArrayImpl(T& cont) : cont_(cont) {cont_.clear();}
  
  void add(JSonSerial& js, JSonClass::Creator* cr, JKString& s) override {
    cont_.resize(cont_.size()+1);
    JObjectPtr* objptr{};
    readArrayValue(js, cont_.back(), objptr, cr, s);
    // TODO: shared pointers when elements in this vector point to an element created in this vector
    if (objptr && objptr->init_) pointers_.push_back(Ptr{cont_.size()-1, objptr});
  }
  
  void end(JSonSerial&) override {
    cont_.shrink_to_fit();
    if (pointers_.size() > 1) fixPointers<T>(cont_, pointers_);
  }
  
  template<class C>
  static void fixPointers(typename std::enable_if<!is_smart_ptr<typename C::value_type>::value,C>::type &,
                          std::vector<Ptr> &) {}
  
  template<class C>
  static void fixPointers(typename std::enable_if<is_smart_ptr<typename C::value_type>::value,C>::type & cont,
                          std::vector<Ptr> & pointers) {
    for (auto& it : pointers) it.jsp->shared_ = &cont[it.index];
  }
  T& cont_;
  std::vector<Ptr> pointers_;
};

}
#endif
